@@ -42,6 +42,9 @@
 #ifndef _TIRPC_SVC_H
 #define _TIRPC_SVC_H
 
+#ifdef USE_RPC_RDMA
+#include <assert.h>
+#endif
 #include <sys/cdefs.h>
 #include <rpc/rpc_msg.h>
 #include <rpc/types.h>
@@ -51,11 +54,13 @@
 #if defined(HAVE_BLKIN)
 #include <blkin/zipkin_c.h>
 #endif
-#ifdef USE_LTTNG_NTIRPC
-#include "lttng/xprt.h"
-#endif
 #include <unistd.h>
 
+#include "lttng/ntirpc_traces.h"
+#if (defined(USE_LTTNG_NTIRPC) || defined(USE_LTTNG)) && !defined(LTTNG_PARSING)
+#include "lttng/generated_traces/xprt.h"
+#endif
+
 typedef struct svc_xprt SVCXPRT;
 
 enum xprt_stat {
@@ -144,6 +149,10 @@ typedef struct svc_init_params {
 	uint32_t channels;
 	int32_t idle_timeout;
 	uint32_t thr_stack_size;
+#if defined(_USE_NFS_RDMA) || defined(USE_RPC_RDMA)
+	uint16_t nfs_rdma_port; /* Shared with Ganesha */
+	uint32_t max_rdma_connections;
+#endif
 } svc_init_params;
 
 /* Svc param flags */
@@ -168,6 +177,7 @@ typedef struct svc_init_params {
 #define SVC_XPRT_FLAG_RELEASING		0x0040	/* (*xp_destroy) was called */
 #define SVC_XPRT_FLAG_UREG		0x0080
 #define SVC_XPRT_TREE_LOCKED		0x0100
+#define SVC_XPRT_FLAG_REMOTE_ADDR_SET	0x0200	/* remote addr was final set */
 
 #define SVC_XPRT_FLAG_DESTROYED (SVC_XPRT_FLAG_DESTROYING \
 				| SVC_XPRT_FLAG_RELEASING)
@@ -257,7 +267,10 @@ struct svc_xprt {
 
 	/* handle incoming connections (per xp_fd) */
 	union {
-		svc_req_fun_t process_cb;
+		struct {
+			svc_req_fun_t process_cb;
+			svc_xprt_fun_t remote_addr_set_cb;
+		};
 		svc_xprt_fun_t rendezvous_cb;
 	}  xp_dispatch;
 	SVCXPRT *xp_parent;
@@ -271,6 +284,11 @@ struct svc_xprt {
 	void *xp_u1;		/* client user data */
 	void *xp_u2;		/* client user data */
 
+#if defined(_USE_NFS_RDMA) || defined(USE_RPC_RDMA)
+	bool xp_rdma;		/* True if this xprt is RDMA enabled.
+				 * Shared with Ganesha */
+#endif
+
 	struct rpc_address xp_local;	/* local address, length, port */
 	struct rpc_address xp_remote;	/* remote address, length, port */
 	struct rpc_address xp_proxy;	/* proxy address, length, port */
@@ -302,6 +320,18 @@ struct svc_xprt {
 	} xp_pktinfo;
 };
 
+#define XPRT_FMT "xprt: [ptr = {},flags = {},fd = {},type = {},refcnt = {}]"
+#define XPRT_VARS(_xprt) (_xprt), (_xprt)->xp_flags, (_xprt)->xp_fd, \
+	(_xprt)->xp_type, (_xprt)->xp_refcnt
+
+#define XPRT_AUTO_TRACEPOINT(_xprt, event, log_level, format, ...) \
+	NTIRPC_AUTO_TRACEPOINT(xprt, event, log_level, \
+		XPRT_FMT " | " format, XPRT_VARS(_xprt), ##__VA_ARGS__)
+
+#define XPRT_UNIQUE_AUTO_TRACEPOINT(_xprt, event, log_level, format, ...) \
+	NTIRPC_UNIQUE_AUTO_TRACEPOINT(xprt, event, log_level, \
+		XPRT_FMT " | " format, XPRT_VARS(_xprt), ##__VA_ARGS__)
+
 /* Service record used by exported search routines */
 typedef struct svc_record {
 	rpcprog_t sc_prog;
@@ -339,6 +369,12 @@ struct svc_req {
 	void *rq_u2;		/* user data */
 	uint64_t rq_cksum;
 
+#if defined(_USE_NFS_RDMA) || defined(USE_RPC_RDMA)
+	/* Data buffer used to server read/readdir from fs */
+	int data_chunk_length;	/* Shared with Ganesha */
+	uint8_t *data_chunk;	/* Shared with Ganesha */
+#endif
+
 	/* Moved in N TI-RPC */
 	struct SVCAUTH *rq_auth;	/* auth handle */
 	void *rq_ap1;		/* auth private */
@@ -417,9 +453,7 @@ __END_DECLS
 static inline void svc_ref_it(SVCXPRT *xprt, u_int flags,
 			      const char *tag, const int line)
 {
-#ifdef USE_LTTNG_NTIRPC
-	int32_t refs =
-#endif /* USE_LTTNG_NTIRPC */
+	const int32_t refs =
 		atomic_inc_int32_t(&xprt->xp_refcnt);
 
 	if (flags & SVC_REF_FLAG_LOCKED)  {
@@ -427,9 +461,9 @@ static inline void svc_ref_it(SVCXPRT *xprt, u_int flags,
 		mutex_unlock(&xprt->xp_lock);
 	}
 	XPRT_TRACE(xprt, __func__, tag, line);
-#ifdef USE_LTTNG_NTIRPC
-	tracepoint(xprt, ref, tag, line, xprt, refs);
-#endif /* USE_LTTNG_NTIRPC */
+
+	XPRT_AUTO_TRACEPOINT(xprt, incref, TRACE_DEBUG,
+		"Incref. refs: {}", refs);
 }
 #define SVC_REF2(xprt, flags, tag, line)				\
 	svc_ref_it(xprt, flags, tag, line)
@@ -457,9 +491,9 @@ static inline void svc_release_it(SVCXPRT *xprt, u_int flags,
 		mutex_unlock(&xprt->xp_lock);
 	}
 	XPRT_TRACE(xprt, __func__, tag, line);
-#ifdef USE_LTTNG_NTIRPC
-	tracepoint(xprt, unref, tag, line, xprt, refs);
-#endif /* USE_LTTNG_NTIRPC */
+
+	XPRT_AUTO_TRACEPOINT(xprt, release_it, TRACE_DEBUG, "Release ref. "
+		"refs={}", refs);
 
 	if (likely(refs > 0)) {
 		/* normal case */
@@ -483,6 +517,7 @@ static inline void svc_release_it(SVCXPRT *xprt, u_int flags,
 #define SVC_RELEASE(xprt, flags)					\
 	svc_release_it(xprt, flags, __func__, __LINE__)
 
+#define SVC_DESTROY_RETRY 10
 /* SVC_DESTROY() is SVC_RELEASE() with once-only semantics.
  * Idempotent SVC_XPRT_FLAG_DESTROYED (bit SVC_XPRT_FLAG_DESTROYING)
  * indicates that more references should not be taken.
@@ -490,20 +525,33 @@ static inline void svc_release_it(SVCXPRT *xprt, u_int flags,
 static inline void svc_destroy_it(SVCXPRT *xprt,
 				  const char *tag, const int line)
 {
+	int retry;
 	uint16_t flags = atomic_postset_uint16_t_bits(&xprt->xp_flags,
 						      SVC_XPRT_FLAG_DESTROYING);
 
 	XPRT_TRACE(xprt, __func__, tag, line);
 
-#ifdef USE_LTTNG_NTIRPC
-	tracepoint(xprt, destroy, tag, line, xprt, flags);
-#endif /* USE_LTTNG_NTIRPC */
+	XPRT_AUTO_TRACEPOINT(xprt, destroy_it, TRACE_DEBUG, "Destroy XPRT");
 
 	if (flags & SVC_XPRT_FLAG_DESTROYING) {
 		/* previously set, do nothing */
 		return;
 	}
 
+	/* A newly created xprt will be 1. inserted into the tree
+	 * of svc_xprt_fd, then 2. get xp_ops initialized. But if
+	 * shutdown happens in between, xprt without xp_ops initialization
+	 * will be in the tree of svc_xprt_fd for destroying.
+	 * Since the initialization of xp_ops will not acuqire the
+	 * current held lock, try RETRY here to wait for the
+	 * initialization to be done
+	 */
+	retry = 0;
+	while (xprt->xp_ops == NULL && retry < SVC_DESTROY_RETRY) {
+		sched_yield();
+		retry += 1;
+	};
+
 	/* unlink before dropping last ref */
 	(*(xprt)->xp_ops->xp_unlink)(xprt, flags, tag, line);
 
@@ -512,24 +560,12 @@ static inline void svc_destroy_it(SVCXPRT *xprt,
 		(*(xprt)->xp_ops->xp_unref_user_data)(xprt);
 	}
 
-	/* Check if the connection was already set as dead or closed,
-	 * If set, let's cleanup and close the FDs, so that FIN-ACK
-	 * could be sent to the client immediately */
-	flags = atomic_postclear_uint16_t_bits(&xprt->xp_flags,
-					       SVC_XPRT_FLAG_CLOSE);
-
-	if ((flags & SVC_XPRT_FLAG_CLOSE)
-	    && xprt->xp_fd != RPC_ANYFD) {
-		XPRT_TRACE(xprt, "WARNING! Connection already closed!",
-				  tag, line);
-		(void)close(xprt->xp_fd);
-		__warnx(TIRPC_DEBUG_FLAG_SVC_VC,
-			"%s: Connection already closed, hence fd %d closed",
-			 __func__, xprt->xp_fd);
-		xprt->xp_fd = RPC_ANYFD;
+	/* Let's shutdown the sockets so that FIN-ACK could be sent to the
+	 * client immediately. */
+	if (xprt->xp_fd != RPC_ANYFD) {
+		(void)shutdown(xprt->xp_fd, SHUT_RDWR);
 		if (xprt->xp_fd_send != RPC_ANYFD)
-			(void)close(xprt->xp_fd_send);
-		xprt->xp_fd_send = RPC_ANYFD;
+			(void)shutdown(xprt->xp_fd_send, SHUT_RDWR);
 	}
 
 	svc_release_it(xprt, SVC_RELEASE_FLAG_NONE, tag, line);
