@@ -29,17 +29,6 @@
 #include <stdbool.h>
 #include <reentrant.h>
 
-#ifdef __APPLE__
-int clock_gettime(clockid_t clock, struct timespec *ts)
-{
-	struct timeval tv;
-	gettimeofday(&tv, NULL);
-	ts->tv_sec = tv.tv_sec;
-	ts->tv_nsec = tv.tv_usec * 1000UL;
-	return 0;
-}
-#endif
-
 #if defined(_WIN32)
 pthread_mutex_t clock_mtx = PTHREAD_MUTEX_INITIALIZER;
 
