@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2009, Sun Microsystems, Inc.
- * Copyright (c) 2012-2017 Red Hat, Inc. and/or its affiliates.
+ * Copyright (c) 2012-2018 Red Hat, Inc. and/or its affiliates.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,7 +37,7 @@
  * Copyright (C) 1984, 1988, Sun Microsystems, Inc.
  */
 
-#include <config.h>
+#include "config.h"
 #include <rpc/rpc.h>
 #include <rpc/types.h>
 #include <rpc/xdr_inline.h>
@@ -111,9 +111,9 @@ xdr_rpcblist_ptr(XDR *xdrs, rpcblist_ptr *rp)
 		 */
 		if (freeing)
 			next = (*rp)->rpcb_next;
-		if (!xdr_reference
-		    (xdrs, (caddr_t *) rp, (u_int) sizeof(rpcblist),
-		     (xdrproc_t) xdr_rpcb)) {
+		if (!xdr_reference(xdrs, (void *) rp,
+				   (u_int) sizeof(rpcblist),
+				   (xdrproc_t) xdr_rpcb)) {
 			return (false);
 		}
 		if (freeing) {
@@ -188,9 +188,9 @@ xdr_rpcb_entry_list_ptr(XDR *xdrs, rpcb_entry_list_ptr *rp)
 		 */
 		if (freeing)
 			next = (*rp)->rpcb_entry_next;
-		if (!xdr_reference
-		    (xdrs, (caddr_t *) rp, (u_int) sizeof(rpcb_entry_list),
-		     (xdrproc_t) xdr_rpcb_entry)) {
+		if (!xdr_reference(xdrs, (void *) rp,
+				   (u_int) sizeof(rpcb_entry_list),
+				   (xdrproc_t) xdr_rpcb_entry)) {
 			return (false);
 		}
 		if (freeing) {
@@ -264,8 +264,7 @@ xdr_rpcb_rmtcallargs(XDR *xdrs, struct rpcb_rmtcallargs *p)
 		return (false);
 	}
 	position = XDR_GETPOS(xdrs);
-	objp->args.args_len =
-	    (u_int) ((u_long) position - (u_long) argposition);
+	objp->args.args_len = position - argposition;
 
 	XDR_SETPOS(xdrs, lenposition);
 	if (!XDR_PUTUINT32(xdrs, objp->args.args_len)) {
@@ -291,7 +290,7 @@ xdr_rpcb_rmtcallres(XDR *xdrs, struct rpcb_rmtcallres *p)
 
 	if (!xdr_string(xdrs, &objp->addr, (u_int) ~ 0))
 		return (false);
-	if (!xdr_u_int(xdrs, &objp->results.results_len))
+	if (!xdr_uint32_t(xdrs, &objp->results.results_len))
 		return (false);
 	dummy = (*(objp->xdr_res)) (xdrs, objp->results.results_val);
 	return (dummy);
