@@ -48,30 +48,152 @@ static struct svc_auth_ops svc_auth_gss_ops;
 #define SVCAUTH_PRIVATE(auth) \
 	((struct svc_rpc_gss_data *)(auth)->svc_ah_private)
 
+#define OLDEST_TIMESTAMP 1
+
+struct svcauth_gss_creds_holder {
+	gss_cred_id_t gss_creds;
+	gss_cred_id_t prev_gss_creds;
+	int64_t gss_creds_expires;
+	/* RW-Lock to synchronise reads and writes to svcauth_gss_creds_holder */
+	rwlock_t gss_creds_lock;
+};
+
+struct svcauth_gss_name_holder {
+	gss_name_t gss_name;
+	/* RW-Lock to synchronise reads and writes to svcauth_gss_name_holder */
+	rwlock_t gss_name_lock;
+};
+
 /* Global server credentials. */
-gss_cred_id_t svcauth_gss_creds;
-static gss_name_t svcauth_gss_name;
-static int64_t svcauth_gss_creds_expires;
-static mutex_t svcauth_gss_creds_lock = MUTEX_INITIALIZER;
-static gss_cred_id_t svcauth_prev_gss_creds;
+static struct svcauth_gss_creds_holder server_creds = {
+	.gss_creds = NULL,
+	.prev_gss_creds = NULL,
+	.gss_creds_expires = OLDEST_TIMESTAMP,
+	.gss_creds_lock = RWLOCK_INITIALIZER
+};
+
+static struct svcauth_gss_name_holder server_gss_name = {
+	.gss_name = NULL,
+	.gss_name_lock = RWLOCK_INITIALIZER
+};
+
+/* Global flag to indicate if svcauth_gss is enabled (default: enabled) */
+bool svcauth_gss_enabled = true;
+
+static mutex_t svcauth_gss_status_lock = MUTEX_INITIALIZER;
+
+/**
+ * @brief This function sets the global svcauth_gss authentication status
+ *
+ * The status can be ON or OFF, depending on the input.
+ * If the new status is OFF, we reset the global auth state variables and
+ * clear gss-context cache.
+ *
+ * We protect the entire function through a mutex to prevent conflicts between
+ * threads trying to set different statuses.
+ *
+ * @note When enabling the status, this function must be called after the
+ * auth-gss name and credentials have been initialised.
+ */
+void
+svcauth_gss_set_status(bool status_enabled)
+{
+	OM_uint32 maj_stat, min_stat;
+
+	/* Acquire mutex to prevent interference by another invocation */
+	mutex_lock(&svcauth_gss_status_lock);
+
+	if (svcauth_gss_enabled == status_enabled) {
+		mutex_unlock(&svcauth_gss_status_lock);
+		__warnx(TIRPC_DEBUG_FLAG_AUTH,
+			"%s: svcauth_gss status is already set to %d",
+			__func__, status_enabled);
+		return;
+	}
+	svcauth_gss_enabled = status_enabled;
+
+	if (status_enabled) {
+		/* No further action required */
+		mutex_unlock(&svcauth_gss_status_lock);
+		return;
+	}
+
+	/* Reset all state related to svcauth_gss credentials */
+	rwlock_wrlock(&server_creds.gss_creds_lock);
+
+	if (server_creds.gss_creds != NULL) {
+		maj_stat = gss_release_cred(&min_stat, &server_creds.gss_creds);
+
+		if (maj_stat != GSS_S_COMPLETE) {
+			__warnx(TIRPC_DEBUG_FLAG_AUTH,
+				"%s: failed to release gss_creds major=%u minor=%u",
+				__func__, maj_stat, min_stat);
+		}
+		server_creds.gss_creds = NULL;
+	}
+	if (server_creds.prev_gss_creds != NULL) {
+		maj_stat = gss_release_cred(&min_stat, &server_creds.prev_gss_creds);
+
+		if (maj_stat != GSS_S_COMPLETE) {
+			__warnx(TIRPC_DEBUG_FLAG_AUTH,
+				"%s: failed to release prev_gss_creds major=%u minor=%u",
+				__func__, maj_stat, min_stat);
+		}
+		server_creds.prev_gss_creds = NULL;
+	}
+	/* Set expiry-time in the past to not use existing creds */
+	server_creds.gss_creds_expires = OLDEST_TIMESTAMP;
+
+	rwlock_unlock(&server_creds.gss_creds_lock);
+
+	/* Reset svcauth_gss service name */
+	rwlock_wrlock(&server_gss_name.gss_name_lock);
+
+	if (server_gss_name.gss_name != NULL) {
+		maj_stat = gss_release_name(&min_stat, &server_gss_name.gss_name);
+		if (maj_stat != GSS_S_COMPLETE) {
+			__warnx(TIRPC_DEBUG_FLAG_AUTH,
+				"%s: failed to release gss_name major=%u minor=%u",
+				__func__, maj_stat, min_stat);
+		}
+		server_gss_name.gss_name = NULL;
+	}
+	rwlock_unlock(&server_gss_name.gss_name_lock);
+
+	/* Clear authgss-context cache */
+	authgss_ctx_hash_clear();
+
+	/* Now release the mutex to permit other threads */
+	mutex_unlock(&svcauth_gss_status_lock);
+}
 
 bool
 svcauth_gss_set_svc_name(gss_name_t name)
 {
 	OM_uint32 maj_stat, min_stat;
 
-	if (svcauth_gss_name != NULL) {
-		maj_stat = gss_release_name(&min_stat, &svcauth_gss_name);
-		if (maj_stat != GSS_S_COMPLETE)
+	/* Acquire write lock before updating server_gss_name.gss_name */
+	rwlock_wrlock(&server_gss_name.gss_name_lock);
+
+	if (server_gss_name.gss_name != NULL) {
+		maj_stat = gss_release_name(&min_stat,
+			&server_gss_name.gss_name);
+		if (maj_stat != GSS_S_COMPLETE) {
+			rwlock_unlock(&server_gss_name.gss_name_lock);
 			return (false);
-		svcauth_gss_name = NULL;
+		}
+		server_gss_name.gss_name = NULL;
 	}
 
 	/* XXX Ganesha */
-	if (svcauth_gss_name == GSS_C_NO_NAME)
+	if (server_gss_name.gss_name == GSS_C_NO_NAME) {
+		rwlock_unlock(&server_gss_name.gss_name_lock);
 		return (true);
+	}
+	maj_stat = gss_duplicate_name(&min_stat, name,
+		&server_gss_name.gss_name);
+	rwlock_unlock(&server_gss_name.gss_name_lock);
 
-	maj_stat = gss_duplicate_name(&min_stat, name, &svcauth_gss_name);
 	if (maj_stat != GSS_S_COMPLETE)
 		return (false);
 
@@ -127,36 +249,56 @@ svcauth_gss_acquire_cred(void)
 	gss_cred_id_t old_creds, ancient_creds;
 
 	now = get_time_fast();
-	if (svcauth_gss_creds && (!svcauth_gss_creds_expires || svcauth_gss_creds_expires > now))
-		return (true);
 
-	mutex_lock(&svcauth_gss_creds_lock);
-	if (svcauth_gss_creds && (!svcauth_gss_creds_expires || svcauth_gss_creds_expires > now)) {
+	rwlock_rdlock(&server_creds.gss_creds_lock);
+	if (server_creds.gss_creds && (!server_creds.gss_creds_expires ||
+		server_creds.gss_creds_expires > now)) {
+		rwlock_unlock(&server_creds.gss_creds_lock);
+		return (true);
+	}
+	rwlock_unlock(&server_creds.gss_creds_lock);
+
+	/* Now acquire write-lock for writing server_creds.gss_creds */
+	rwlock_wrlock(&server_creds.gss_creds_lock);
+
+	if (server_creds.gss_creds && (!server_creds.gss_creds_expires ||
+		server_creds.gss_creds_expires > now)) {
 		maj_stat = GSS_S_COMPLETE;
 	} else {
-		ancient_creds = svcauth_prev_gss_creds;
-		old_creds = svcauth_gss_creds;
+		ancient_creds = server_creds.prev_gss_creds;
+		old_creds = server_creds.gss_creds;
 		timerec = 0;
 		now = get_time_fast();
-		maj_stat =
-		    gss_acquire_cred(&min_stat, svcauth_gss_name, 0, GSS_C_NULL_OID_SET,
-				     GSS_C_ACCEPT, &svcauth_gss_creds, NULL, &timerec);
+
+		/* Acquire read-lock before reading server_gss_name.gss_name */
+		rwlock_rdlock(&server_gss_name.gss_name_lock);
+		maj_stat = gss_acquire_cred(&min_stat, server_gss_name.gss_name,
+			0, GSS_C_NULL_OID_SET, GSS_C_ACCEPT,
+			&server_creds.gss_creds, NULL, &timerec);
+		rwlock_unlock(&server_gss_name.gss_name_lock);
+
 		if (maj_stat == GSS_S_COMPLETE) {
 			if (timerec == GSS_C_INDEFINITE)
-				svcauth_gss_creds_expires = 0;
+				server_creds.gss_creds_expires = 0;
 			else
-				svcauth_gss_creds_expires = now + timerec;
+				server_creds.gss_creds_expires = now + timerec;
 			if (old_creds) {
-				svcauth_prev_gss_creds = old_creds;
+				server_creds.prev_gss_creds = old_creds;
 			}
 			if (ancient_creds) {
 				(void) gss_release_cred(&min_stat, &ancient_creds);
 			}
 		}
 	}
-	mutex_unlock(&svcauth_gss_creds_lock);
-	if (maj_stat != GSS_S_COMPLETE)
+	/* Release the server_creds.gss_creds write-lock */
+	rwlock_unlock(&server_creds.gss_creds_lock);
+
+	if (maj_stat != GSS_S_COMPLETE) {
+		__warnx(TIRPC_DEBUG_FLAG_AUTH,
+			"%s: gss_acquire_cred failed major=%u minor=%u", __func__,
+			maj_stat, min_stat);
 		return (false);
+	}
 
 	return (true);
 }
@@ -164,18 +306,22 @@ svcauth_gss_acquire_cred(void)
 bool
 svcauth_gss_release_cred(void)
 {
+	rwlock_wrlock(&server_creds.gss_creds_lock);
 #if 0
 	OM_uint32 maj_stat, min_stat;
 
-	maj_stat = gss_release_cred(&min_stat, &svcauth_gss_creds);
-	if (maj_stat != GSS_S_COMPLETE)
+	maj_stat = gss_release_cred(&min_stat, &server_creds.gss_creds);
+	if (maj_stat != GSS_S_COMPLETE) {
+		rwlock_unlock(&server_creds.gss_creds_lock);
 		return (false);
-	svcauth_gss_creds = NULL;
+	}
+	server_creds.gss_creds = NULL;
 #else
-	svcauth_gss_creds_expires = 1;
+	server_creds.gss_creds_expires = OLDEST_TIMESTAMP;
 	/* make any future callers to svcauth_gss_acquire_cred() should get a new cred. */
 #endif
 
+	rwlock_unlock(&server_creds.gss_creds_lock);
 	return (true);
 }
 
@@ -204,11 +350,20 @@ svcauth_gss_accept_sec_context(struct svc_req *req,
 		return (false);
 	}
 
-	gr->gr_major =
-	    gss_accept_sec_context(&gr->gr_minor, &gd->ctx, svcauth_gss_creds,
-				   &recv_tok, GSS_C_NO_CHANNEL_BINDINGS,
-				   &gd->client_name, &mech, &gr->gr_token,
-				   &ret_flags, &time_rec, NULL);
+	rwlock_rdlock(&server_creds.gss_creds_lock);
+
+	/* We can not accept incoming context, if server gss-creds are NULL. */
+	if (server_creds.gss_creds == NULL) {
+		rwlock_unlock(&server_creds.gss_creds_lock);
+		xdr_free((xdrproc_t)xdr_rpc_gss_init_args, (void *)&recv_tok);
+		return false;
+	}
+
+	gr->gr_major = gss_accept_sec_context(&gr->gr_minor, &gd->ctx,
+		server_creds.gss_creds, &recv_tok, GSS_C_NO_CHANNEL_BINDINGS,
+		&gd->client_name, &mech, &gr->gr_token,
+		&ret_flags, &time_rec, NULL);
+	rwlock_unlock(&server_creds.gss_creds_lock);
 
 	xdr_free((xdrproc_t)xdr_rpc_gss_init_args, (void *)&recv_tok);
 
@@ -230,13 +385,12 @@ svcauth_gss_accept_sec_context(struct svc_req *req,
 	/* ANDROS: change for debugging linux kernel version...
 	   gr->gr_win = 0x00000005;
 	 */
-	gr->gr_win = sizeof(gd->seqmask) * 8;
+	gr->gr_win = SVC_GSS_SEQ_WIN;
 
 	/* Save client info. */
 	gd->sec.mech = mech;
 	gd->sec.qop = GSS_C_QOP_DEFAULT;
 	gd->sec.svc = gc->gc_svc;
-	gd->win = gr->gr_win;
 
 	if (time_rec == GSS_C_INDEFINITE) time_rec = INDEF_EXPIRE;
 	if (time_rec > 10) time_rec -= 5;
@@ -385,6 +539,65 @@ svcauth_gss_nextverf(struct svc_req *req, struct svc_rpc_gss_data *gd,
 	return (true);
 }
 
+static bool
+svcauth_gss_is_seq_num_valid(struct svc_rpc_gss_data *gd, u_int32_t seq_num)
+{
+	int next_unseen_seq_num_bit;
+	int seq_num_bit_in_window = seq_num % SVC_GSS_SEQ_WIN_INTERNAL;
+
+	/* Handle sequence number lesser than the max seen sequence number */
+	if (seq_num < gd->seqlast) {
+
+		/* Return false if current sequence number is outside the
+		 * window.
+		 */
+		if ((gd->seqlast - seq_num) >= SVC_GSS_SEQ_WIN_INTERNAL) {
+			__warnx(TIRPC_DEBUG_FLAG_AUTH,
+				"%s: sequence number: %u is outside the sequence window. Max seen sequence number: %u Sequence window: %d",
+				__func__, seq_num, gd->seqlast,
+				SVC_GSS_SEQ_WIN_INTERNAL);
+			return false;
+		}
+
+		/* Return false if current sequence number is already seen */
+		if (isset(gd->seqmask, seq_num_bit_in_window)) {
+			__warnx(TIRPC_DEBUG_FLAG_AUTH,
+				"%s: sequence number: %u is already seen. ",
+				__func__, seq_num);
+			return false;
+		}
+
+		/* Set the bit to mark the current sequence number as seen */
+		setbit(gd->seqmask, seq_num_bit_in_window);
+		return true;
+	}
+
+	/* Handle sequence number greater than the max seen sequence number */
+
+	/* Unset the sequence mask if current sequence number is greater than
+	 * the max seen sequence number by N (where N >= sequence-window)
+	 */
+	if ((seq_num - gd->seqlast) >= SVC_GSS_SEQ_WIN_INTERNAL) {
+		memset(gd->seqmask, 0, sizeof(gd->seqmask));
+		gd->seqlast = seq_num;
+	} else {
+		/* In this case, we clear the next unseen sequence number bits
+		 * upto the new max sequence number
+		 */
+		while (gd->seqlast < seq_num) {
+			gd->seqlast++;
+			next_unseen_seq_num_bit =
+				gd->seqlast % SVC_GSS_SEQ_WIN_INTERNAL;
+			clrbit(gd->seqmask, next_unseen_seq_num_bit);
+		}
+	}
+
+	/* Set the bit to mark the current sequence number as seen */
+	setbit(gd->seqmask, seq_num_bit_in_window);
+	return true;
+}
+
+
 enum auth_stat
 _svcauth_gss(struct svc_req *req, bool *no_dispatch)
 {
@@ -393,7 +606,7 @@ _svcauth_gss(struct svc_req *req, bool *no_dispatch)
 	struct svc_rpc_gss_data *gd = NULL;
 	struct rpc_gss_cred *gc = NULL;
 	struct rpc_gss_init_res gr;
-	int call_stat, offset;
+	int call_stat;
 	OM_uint32 min_stat;
 	enum auth_stat rc = AUTH_OK;
 
@@ -480,19 +693,10 @@ _svcauth_gss(struct svc_req *req, bool *no_dispatch)
 			 goto gd_free;
 		}
 
-		/* XXX implied serialization?  or just fudging?  advance if
-		 * greater? */
-		offset = gd->seqlast - gc->gc_seq;
-		if (offset < 0) {
-			gd->seqlast = gc->gc_seq;
-			offset = 0 - offset;
-			gd->seqmask <<= offset;
-			offset = 0;
-		} else if (offset >= gd->win || (gd->seqmask & (1 << offset))) {
-			*no_dispatch = true;
+		*no_dispatch = !svcauth_gss_is_seq_num_valid(gd, gc->gc_seq);
+		if (*no_dispatch) {
 			goto gd_free;
 		}
-		gd->seqmask |= (1 << offset);	/* XXX harmless */
 
 		req->rq_ap1 = (void *)(uintptr_t) gc->gc_seq; /* GCC casts */
 		req->rq_clntname = (char *) gd->client_name;
@@ -511,6 +715,13 @@ _svcauth_gss(struct svc_req *req, bool *no_dispatch)
 			goto gd_free;
 		}
 
+		/* Fail new INIT / CONTINUE_INIT requests if svcauth_gss is disabled */
+		if (!svcauth_gss_enabled) {
+			__warnx(TIRPC_DEBUG_FLAG_AUTH, "%s: auth-gss disabled. Failing", __func__);
+			rc = AUTH_REJECTEDCRED;
+			goto gd_free;
+		}
+
 		/* XXX why unconditionally acquire creds? */
 		if (!svcauth_gss_acquire_cred()) {
 			rc = AUTH_FAILED;
